{===================================================================================================
Unit                : UCA_CL_Bitmap_Ress_Loader
Klasse              :
Datum               : 28.01.2022
Autor               : Uwe Rimarski
Zweck               : Dynamisches Laden von BITMAP Ressourcen (Ohne {$R xxx.res} Anweisung
Hinweise            : Bei mehreren Ressourcen Loadern sollte eine BASIS Klasse (Interface) angelegt werden
===================================================================================================}
UNIT UCA_CL_Bitmap_Ress_Loader;
{$mode objfpc}{$H+}
INTERFACE
USES
    Classes
  , Sysutils
  , Resource
  , ResReader
  , Graphics
  ;

TYPE
  TCL_CA_Bitmap_Ressourcen = CLASS( TObject )
    PRIVATE
      mo_Stream    : TFileStream;
      mo_Ressource : TResources;
    PROTECTED

      PROPERTY    Stream : TFileStream
                  READ   mo_Stream
                  WRITE  mo_Stream;

      PROPERTY    Ressource : TResources
                  READ   mo_Ressource
                  WRITE  mo_Ressource;

    PUBLIC
      CONSTRUCTOR Create;
      DESTRUCTOR  Destroy;  OVERRIDE;

    PUBLIC
      FUNCTION    LOAD_FromRessource_File( iv_FileName : String ) : Boolean;
      FUNCTION    LOAD_BitMap( iv_Name : String; co_BitMap : TBitMap ) : Boolean;
    PUBLIC
      PROCEDURE   LOAD_Names_FromRessource( io_Strings : TStrings; iv_ListClearing : Boolean = True );

  END ;


IMPLEMENTATION
{===================================================================================================
===================================================================================================}
CONSTRUCTOR TCL_CA_Bitmap_Ressourcen.Create;
BEGIN
  INHERITED Create;
  mo_Stream    := NIL;
  mo_Ressource := TResources.Create;
END ;



{===================================================================================================
===================================================================================================}
DESTRUCTOR TCL_CA_Bitmap_Ressourcen.Destroy;
BEGIN
  IF Assigned( Stream ) THEN
    mo_Stream.Destroy;
  IF Assigned( Ressource ) THEN
    mo_Ressource.Destroy;
  INHERITED Destroy;
END ;



{===================================================================================================
===================================================================================================}
FUNCTION TCL_CA_Bitmap_Ressourcen.LOAD_BitMap( iv_Name : String; co_BitMap : TBitMap ) : Boolean;
VAR lv_i : Integer; lo_Item : TAbstractResource; lo_Pic : TPortableNetworkGraphic;
BEGIN
  FOR lv_i := 0 TO Ressource.Count- 1 DO BEGIN
    lo_Item := mo_Ressource.Items[ lv_i ];
    IF lo_Item.Name.Name = iv_Name THEN BEGIN
      lo_Pic := TPortableNetworkGraphic.Create;
      lo_Pic.LoadFromStream( lo_Item.RawData );
      co_BitMap.Assign( lo_Pic );
      lo_Pic.Destroy;
      Result := True;
      EXIT;
    END;
  END;
  Result := False;
END ;


{===================================================================================================
===================================================================================================}
PROCEDURE TCL_CA_Bitmap_Ressourcen.LOAD_Names_FromRessource( io_Strings : TStrings; iv_ListClearing : Boolean = True );
VAR lv_i : Integer; lo_Item : TAbstractResource;
BEGIN
  IF iv_ListClearing THEN
    io_Strings.Clear;
  FOR lv_i := 0 TO mo_Ressource.Count- 1 DO BEGIN
    lo_Item := mo_Ressource.Items[ lv_i ];
    io_Strings.Add( lo_Item.Name.Name );
  END ;
END ;



{===================================================================================================
===================================================================================================}
FUNCTION TCL_CA_Bitmap_Ressourcen.LOAD_FromRessource_File( iv_FileName : String ) : Boolean;
VAR lo_Reader: TAbstractResourceReader;
BEGIN
  Result := False;
  IF Assigned( mo_Stream ) THEN
    mo_Stream.Destroy;
  mo_Stream := TFileStream.Create( iv_FileName, fmOpenRead);
  IF NOT Assigned( mo_Stream ) THEN EXIT;
  TRY
    lo_Reader := TResources.FindReader( mo_Stream );
    IF NOT Assigned( lo_Reader ) THEN EXIT;
    mo_Ressource.LoadfromStream( mo_Stream, lo_Reader );
  FINALLY;
    lo_Reader.Destroy;
  END ;
  Result := True;
END ;




END .

